package cc.fascinated.common;

import cc.fascinated.exception.impl.BadRequestException;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.List;
import java.util.function.Supplier;

/**
 * @author Fascinated (fascinated7)
 */
@Getter
public class PaginationBuilder<T> {
    /**
     * The number of items per page.
     */
    private int itemsPerPage;

    /**
     * The items to paginate.
     */
    private List<T> items;

    /**
     * Sets the number of items per page.
     *
     * @param itemsPerPage The number of items per page.
     * @return The pagination builder.
     */
    public PaginationBuilder<T> itemsPerPage(int itemsPerPage) {
        this.itemsPerPage = itemsPerPage;
        return this;
    }

    /**
     * Sets the items to paginate.
     *
     * @param getItems The items to paginate.
     * @return The pagination builder.
     */
    public PaginationBuilder<T> fillItems(Supplier<List<T>> getItems) {
        this.items = getItems.get();
        return this;
    }

    /**
     * Builds the pagination.
     *
     * @return The pagination.
     */
    public PaginationBuilder<T> build() {
        return new PaginationBuilder<>();
    }

    /**
     * Gets a page of items.
     *
     * @param page     The page number.
     * @return The page.
     */
    public Page<T> getPage(int page) {
        int totalItems = this.items.size();
        int totalPages = (int) Math.ceil((double) totalItems / this.itemsPerPage);

        if (page < 1 || page > totalPages) {
            throw new BadRequestException("Invalid page number");
        }
        List<T> items = this.items.subList((page - 1) * this.itemsPerPage, Math.min(page * this.itemsPerPage, totalItems));
        return new Page<>(
                items,
                new Page.Metadata(page, totalPages, totalItems)
        );
    }

    @AllArgsConstructor
    @Getter
    public static class Page<T> {
        /**
         * The items on the page.
         */
        private final List<T> items;

        /**
         * The metadata of the page.
         */
        private final Metadata metadata;

        @AllArgsConstructor
        @Getter
        public static class Metadata {
            /**
             * The page number.
             */
            private final int page;

            /**
             * The total number of pages.
             */
            private final int totalPages;

            /**
             * The total number of items.
             */
            private final int totalItems;
        }
    }
}
